=begin EltenAppInfo
Name=Showdown game
Version=1.0
Author=grzezlo
=end EltenAppInfo
class Grzezlo_Eltendev_showdown < Program_AppDevShowdown
  TABLE_WIDTH = 7
  TABLE_LENGTH = 14

  class GameSound
    MIN_VOLUME = 0.1
    def self.set_listener(listener)
      @@listener = listener
    end

    def self.set_samples_dir(dir)
      @@samples_dir = dir
    end

    def self.set_space_size(width, length)
      @@space_width = width
      @@space_length = length
    end

    def initialize(file_name, owner, loop = false)
      @so = Sound.new(@@samples_dir + file_name + ".ogg", 1, loop)
      @owner = owner
    end # initialize

    def stop
      @so.stop if @so.playing
    end

    def play
      stop
      @so.position = 0
      @so.play
    end

    def update_position
      x_diff = @owner.x - @@listener.x
      x_diff = x_diff / (@@space_width - 1).to_f
      @so.pan = x_diff
      y_diff = (@@listener.y - @owner.y).abs
      y_diff = y_diff / (@@space_length - 1).to_f
      y_diff = (1 - MIN_VOLUME) * y_diff
      y_diff = 1 - y_diff
      @so.volume = y_diff
    end #update_position
  end #class GameSound

  class GameTable
    attr_reader :width, :length

    def initialize(width, length)
      @width = width
      @length = length
      GameSound.set_space_size(width, length)
    end #initialize
  end #GameTable

  class GameObject
    attr_reader :x, :y, :sounds

    def initialize(table, x = 0, y = 0)
      @table = table
      @sounds = {}
      set_position(x, y)
    end

    def new_sound(file_name, loop = false)
      so = GameSound.new(file_name, self, loop)
      so.update_position
      @sounds[file_name] = so
      return so
    end #new_sound

    def update_sounds
      for name, obj in @sounds
        obj.update_position
      end
    end #update_sounds

    def set_position(x, y)
      @x = x
      @y = y
      update_sounds
    end
  end #class GameObject

  def main
    GameSound.set_samples_dir(appfile)
    table = GameTable.new(TABLE_WIDTH, TABLE_LENGTH)
    listener = GameObject.new(table, 1, 1)
    owner = GameObject.new(table, table.width, table.length)
    GameSound.set_listener(listener)
    owner.new_sound("ball", true)
    owner.sounds["ball"].play
    move_sound = owner.new_sound("move")
    table.width.downto(1) { |i|
      move_sound.play
      owner.set_position(i, table.length)
      delay(0.5)
    }
    owner.sounds["ball"].stop
  end #main
end #class

Grzezlo_Eltendev_showdown.new.main
